comment * 

   This program calculates the CRC-32 (PKZIP, etc) value for the file
   specified on the command line. The crc32 procedure implements a
   relatively straightforward bit-by-bit algorithm.

   The "standard" CRC-32 parameters are:

      Width = 32 (order of poly)
      Poly  = 04C11DB7h (truncated poly)
      Init  = 0FFFFFFFFh (initial register value)
      RefIn = True (reflect bits in input bytes)
      RefOut= True (reflect bits in output prior to XOR)
      XorOut= 0FFFFFFFFh (XOR with final output)
      Check = CBF43926h (correct CRC for "123456789")

   To obtain the correct final CRC value with a minimum number of
   instructions in the bit loop, the "non-direct" initial register
   value (46AF6449h) has been substituted for the standard value
   and, for non-augmented byte streams, the remainder run through
   an additional conversion before the final reflection and XOR.
   For augmented byte streams (i.e. with 32 zero bits appended to
   the end of the stream) comment out the conversion and pass a
   length value that includes the appended bits.

;  *

   .486
   .model flat, stdcall
   option casemap :none

   include \masm32\include\windows.inc
   include \masm32\include\user32.inc
   include \masm32\include\kernel32.inc
   include \masm32\include\masm32.inc
   include \masm32\macros\macros.asm
   includelib \masm32\lib\masm32.lib
   includelib \masm32\lib\user32.lib
   includelib \masm32\lib\kernel32.lib

   crc32 PROTO :DWORD,:DWORD

.data

.code
start:

   clock = 1
   call main
   call getc
   exit
   
; 
; This is a modified version of the main procedure from Hutch's SCAN7.ASM.
; 
main proc

   LOCAL fname :DWORD
   LOCAL pMem  :DWORD
   LOCAL tc    :DWORD
   LOCAL buffer[260]:BYTE
   mov fname, ptr$(buffer)    ; get pointer to buffer
   invoke GetCL,1, fname      ; get 1st command line argument

   invoke exist,fname         ; check if the command line file exists
   cmp eax, 0
   je not_exist               ; display error if not

   mov pMem, InputFile(fname) ; load the file into memory

   IF clock
      invoke GetTickCount
      mov tc, eax
   ENDIF

   invoke crc32,pMem,FUNC(filesize,fname)
   push  eax

   IF clock
      invoke GetTickCount
      sub eax, tc
      fn MessageBox,0,str$(eax),"MS timing",MB_OK
   ENDIF

   pop   edx
   invoke dw2hex,edx,addr buffer
   print addr buffer

   jmp pexit

   not_exist:

      print chr$("Cannot find that file",13,10)
      print chr$("SYNTAX: CRC-32 filename.ext",13,10)

   pexit:

      free pMem            ; free the source memory
      
   ret
      
main endp

; 

align 4

crc32 proc uses esi pMem:DWORD,memlen:DWORD

   ; These correct for THIS implementation and CRC-32.
   poly     EQU 04C11DB7h
   init     EQU 46AF6449h
   xorout   EQU 0FFFFFFFFh

   mov   esi,pMem
   mov   ecx,memlen

   ; Use EDX for the working register.
   mov   edx,init

   byteLoop:
      ; Load the next byte into AH.
      mov   ah,[esi]
      ; Reflect AH into AL.
      REPEAT 8
         rcl   ah,1
         rcr   al,1
      ENDM
      ; Start of unrolled bit loop.
      REPEAT 8
         ; Shift next bit into CF.
         shl   al,1
         ; Shift CF into working register.
         rcl   edx,1
         ; Shifted-out bit now in CF, if was 0 then skip XOR.
         jnc   @F
         xor   edx,poly
      @@:
      ENDM
      inc   esi
      dec   ecx
   jnz   byteLoop

   ; This conversion for non-augmented byte streams only.
   REPEAT 32
      shl   edx,1
      jnc   @F
      xor   edx,poly
   @@:
   ENDM

   ; Reflect bits in EDX into EAX.
   REPEAT 32
      rcl   edx,1
      rcr   eax,1
   ENDM

   ; Do final XOR.
   xor   eax,xorout

   ; Return CRC value in EAX.
   ret

crc32 endp

; 

convert proc

   ; This procedure shows how the "non-direct" initial register
   ; value is derived from the standard initial register value.

   mov   eax,0FFFFFFFFh
   mov   ecx,32

   bitLoop:
      mov   edx,1
      and   edx,eax
      jz    @F
      xor   eax,poly
   @@:
      shr   eax,1
      or    edx,edx
      jz    @f
      or    eax,80000000h
   @@:
      dec   ecx
   jnz   bitLoop

   ret

convert endp

; 

getc proc

   ; Waits for and reads a single character from stdin (no echo).

   LOCAL hInput :DWORD
   LOCAL char   :DWORD
   LOCAL bRead  :DWORD
   invoke GetStdHandle,STD_INPUT_HANDLE
   mov hInput,eax
   invoke FlushConsoleInputBuffer,hInput
   invoke SetConsoleMode,hInput,ENABLE_PROCESSED_INPUT
   invoke ReadConsole,hInput,ADDR char,1,ADDR bRead,NULL
   invoke SetConsoleMode,hInput,ENABLE_LINE_INPUT or \
                                ENABLE_ECHO_INPUT or \
                                ENABLE_PROCESSED_INPUT
   mov eax,char
   ret

getc endp

end start